# encoding: utf-8
"""Загрузчик данных из шаблонов."""

from __future__ import unicode_literals

from m3_django_compat import atomic


class ImportException(Exception):
    """Базовое исключение."""


#: Режимы загрузки
STRICT = 1  #: Строгий режим - откат всех изменений при ошибке
SKIP_ERRORS = 2  #: Пропуск ошибок, загрузка валидных строк
DRY_RUN = 3  #: Сбор ошибок и откат изменений


class ImportStrategyException(ImportException):
    """Исключение режима загрузки."""

    def __init__(self, errors, *args, **kwargs):
        super(ImportStrategyException, self).__init__(*args, **kwargs)
        self.errors = errors


def import_file(file_path, config, parser_params=None, loader_params=None):
    """Выполняет импорт файла используя конфигурацию импорта."""

    logger = config.logger()
    rules = config.rules()
    parser = config.parser(
        file_path, rules, logger, **parser_params or {}
    )

    try:
        with atomic():
            loader = config.loader(logger, loader_params)
            loader.load(parser.get_rows())

            errors = logger.messages

            mode = config.mode
            if mode is DRY_RUN or mode is STRICT and errors:
                # Откат транзакции в случае, когда ошибки недопустимы, или
                # в режиме сбора ошибок
                raise ImportStrategyException(errors)

    except ImportStrategyException as exception:
        errors = exception.errors

    return errors
